<?php

  // ###################################################################
  // Function: Get list of subdirectories and files in a given directory
  // ###################################################################

  // Arguments: list-directory (as string)
  //            skip scripts flag; if set .php, .pl, .pm and .cgi files are not included in the list
  // Return:    subdirectories list (as associative array)
  //            files list (as associative array)
  // Hidden items (Unix .filename items) and log files will be ignored!
  // Lists will be sorted on subdirectory/file name; case-sensitive sort (uppercase before lowercase)

  function getFileList($dir, $skipscripts) {
    // ##### Custom sort function for usort #####
    function sort_it($a, $b) {
      return ($a["name"] <= $b["name"]) ? -1 : 1;                                        // sort on subdirectory/file name
    }
    // ###########################################
    $subdirlist = []; $filelist = [];
    if(substr($dir, -1) != "/") {                                                        // add trailing slash if missing
      $dir .= "/";
    }
    // Open pointer to directory and read list of files
    $d = @dir($dir) or die("getFileList: Failed opening directory {$dir} for reading");
    while(FALSE !== ($entry = $d->read())) {
      if($entry{0} == ".") {                                                             // skip hidden entries
        continue;
      }
      // Subdirectories
      if(is_dir("{$dir}{$entry}")) {
        $subdirlist[] = [
          'path' => "{$dir}{$entry}/",
          'name' => "{$entry}",
          'type' => filetype("{$dir}{$entry}"),
          'size' => 0,
          'date' => filemtime("{$dir}{$entry}")
        ];
      }
      // Files
      elseif(is_readable("{$dir}{$entry}")) {
        if (!(stripos($entry, 'error') === FALSE && stripos($entry, 'log') === FALSE)) { // ignore error and other log files (in particular PHP error log)
          continue;
        }
        if ($skipscripts) {                                                              // if skip scripts flag is set, ignore script files
          $fileext = substr($entry, strrpos($entry, '.'));
          if ($fileext == '.php' || $fileext == '.pl' || $fileext == '.pm' || $fileext == '.cgi') {
            continue;
          }
        }
        $filelist[] = [
          'path' => "{$dir}{$entry}",
          'name' => "{$entry}",
          'type' => mime_content_type("{$dir}{$entry}"),
          'size' => filesize("{$dir}{$entry}"),
          'date' => filemtime("{$dir}{$entry}")
        ];
      }
    }
    $d->close();
    usort($subdirlist, "sort_it");
    usort($filelist, "sort_it");
    return array($subdirlist, $filelist);
  }

  // ######################################################################
  // Function: Display list of directory subdirectories/files as HTML table
  // ######################################################################

  // Arguments: subdirectories (as associative array)
  //            files (as associative array)
  //            file descriptions (as associative array)
  //            ignore filename suffix flag; if set, "_xxx..." at end of filename is removed for file description lookup
  // Return:    0 if directory is empty, else 1

  function displayFileList($subdirlist, $filelist, $filedescription, $suffixignore) {
    $phpdir = $_SERVER['DOCUMENT_ROOT'] . '/php/';
    include("{$phpdir}icons.php");                                                       // file-type icon URLs
    if (count($subdirlist) == 0 && count($filelist) == 0) {                              // empty directory
      $ret = 0;
    }
    // List directory as HTML table
    else {
      $ret = 1;
      echo "<table border=\"0\" class=\"small\">\n";
      // Display 'parent directory' link
      echo "<tr>\n";
      echo "<td><img src=\"{$icon['PARENT']}\"/></td>";
      echo "<td width=\"10px\">&nbsp;</td>";
      echo "<td colspan=\"7\"><a href=\"../\">Parent directory</a></td>\n";
      echo "</tr>\n";
      echo "<tr><td colspan=\"9\">&nbsp;</td></tr>";
      // Display subdirectories
      if (count($subdirlist) != 0) {
        foreach($subdirlist as $subdir) {
          $name = $subdir['name'];
          $url  = str_replace($_SERVER['DOCUMENT_ROOT'], '', $name) . '/';               // directory URL (for link)
          $type = 'DIRECTORY';
          $description = '&nbsp;';
          $date = substr(date('r', $subdir['date']), 5, strlen($subdir['date']) - 25);
          $size = '&nbsp;';
          echo "<tr>\n";
          echo "<td><img src=\"{$icon[$type]}\"/></td>";                                 // directory icon
          echo "<td width=\"10px\">&nbsp;</td>";
          echo "<td><a href=\"{$url}\">{$name}</a></td>\n";                              // subdirectory name (and link)
          echo "<td width=\"20px\">&nbsp;</td>";
          echo "<td>{$description}</td>\n";                                              // subdirectory description (blank)
          echo "<td width=\"20px\">&nbsp;</td>";
          echo "<td align=\"right\">{$size}</td>\n";                                     // subdirectory size (actually blank)
          echo "<td width=\"20px\">&nbsp;</td>";
          echo "<td>{$date}</td>\n";                                                     // subdirectory modification date
          echo "</tr>\n";
        }
        echo "<tr><td>&nbsp;</td></tr>";                                                 // blank line between subdirectories and files
      }
      // Display files
      if (count($filelist) != 0) {
        foreach($filelist as $file) {
          $name = $file['name'];
          $url  = str_replace($_SERVER['DOCUMENT_ROOT'], '', $name);                     // file URL (for link)
          $name = substr($name, 0, strrpos($name, '.'));                                 // remove file extension
          $lookupname = $name;                                                           // filename used for file description lookup
          if ($suffixignore) {                                                           // ignore filename's final _xxx... if flag tells so
            if (strpos($lookupname, '_') != 0) {
              $lookupname = substr($lookupname, 0, strrpos($name, '_'));
            }
          }
          $type = $file['type'];
          $date = substr(date('r', $file['date']), 5, strlen($file['date']) - 25);
          $size = $file['size'];
          if ($size <= 1000) {                                                           // transform filesize to kb if > 1K
            $size .= ' bytes &nbsp;';
          }
          else {
            $size = round($file['size'] / 1000) . ' kbytes';
          }
          echo "<tr>\n";
          if (!array_key_exists($type, $icon)) {                                         // lookup file-type icon
            $type = 'FILE';                                                              // default icon if file-type not found
          }
          echo "<td><img src=\"{$icon[$type]}\"/></td>";                                 // file-type icon
          echo "<td width=\"10px\">&nbsp;</td>";
          echo "<td><a href=\"{$url}\">{$name}</a></td>\n";                              // file name (and link)
          echo "<td width=\"20px\">&nbsp;</td>";
          if (array_key_exists($lookupname, $filedescription)) {                         // lookup file description
            $description = $filedescription[$lookupname];
          }
          else {
            $description = '&nbsp;';
          }
          echo "<td>{$description}</td>\n";                                              // file description (or blank if none found)
          echo "<td width=\"20px\">&nbsp;</td>";
          echo "<td align=\"right\">{$size}</td>\n";                                     // file size
          echo "<td width=\"20px\">&nbsp;</td>";
          echo "<td>{$date}</td>\n";                                                     // file modification date
          echo "</tr>\n";
        }
      }
      echo "</tbody>\n";
      echo "</table>\n\n";
    }
    return $ret;
  }

?>
